<template>
	<ywDialog
		v-model="dialogIsShow"
		:headerIcon="dialogHeaderIcon"
		:title="dialogTitle"
		width="680"
		@dlgClosed="closeDialog"
		@submit="submitFormValue"
	>
		<el-form :model="dialogFormValue" ref="dialogFormRef" :rules="dialogFormRules" :label-width="isEditDialog ? 54 : 78">
			<el-row :gutter="35">
				<template v-if="!isEditDialog">
					<el-col :span="12" class="mb20">
						<el-form-item label="用户ID" prop="user_name">
							<el-input v-model="dialogFormValue.user_name"></el-input>
						</el-form-item>
					</el-col>
					<el-col :span="12" class="mb20"
						><el-form-item label="密码" prop="password" auto-complete="new-password" label-width="54">
							<el-input type="password" v-model="dialogFormValue.password"></el-input>
						</el-form-item>
					</el-col>
					<el-col :span="12" class="mb20"
						><el-form-item label="确认密码" prop="confirmPassword" auto-complete="new-password">
							<el-input type="password" v-model="dialogFormValue.confirmPassword"></el-input>
						</el-form-item>
					</el-col>
				</template>
				<el-col :span="12" class="mb20"
					><el-form-item label="电话" prop="phone" label-width="54">
						<el-input v-model="dialogFormValue.phone"></el-input>
					</el-form-item>
				</el-col>
				<el-col :span="12" class="mb20">
					<el-form-item label="中文名" prop="real_name">
						<el-input v-model="dialogFormValue.real_name"></el-input>
					</el-form-item>
				</el-col>
				<el-col :span="12" class="mb20">
					<el-form-item label="部门" prop="part" label-width="54">
						<el-input v-model="dialogFormValue.part"></el-input>
					</el-form-item>
				</el-col>
				<el-col :span="12" class="mb20"
					><el-form-item label="性别" prop="sex">
						<el-select v-model="dialogFormValue.sex">
							<el-option v-for="item in Object.keys(userSexMap)" :key="item" :value="item" :label="userSexMap[item]"></el-option>
						</el-select>
					</el-form-item>
				</el-col>
				<el-col :span="12" class="mb20">
					<el-form-item label="邮箱" prop="email" label-width="54">
						<el-input v-model="dialogFormValue.email"></el-input>
					</el-form-item>
				</el-col>
				<template v-if="!isEditDialog"
					><el-col :span="12" class="mb20"
						><el-form-item label="角色" prop="user_roles">
							<el-select class="w100" placeholder="请选择角色" filterable multiple v-model="dialogFormValue.user_roles">
								<el-option v-for="(item, index) in roleList" :key="index + ''" :value="item.id" :label="item?.title"></el-option>
							</el-select>
						</el-form-item>
					</el-col>
				</template>

				<el-col :span="24" class="mb20">
					<el-form-item label="说明" prop="note">
						<el-input placeholder="请输入说明" v-model="dialogFormValue.note" type="textarea" :rows="3" /> </el-form-item
				></el-col>
			</el-row>
		</el-form>
	</ywDialog>
</template>

<script setup lang="ts">
import ywDialog from '/@/components/dialog/yw-dialog.vue';

import type { FormInstance, FormRules } from 'element-plus';
import { ElMessage } from 'element-plus';

import { computed, ref, watch } from 'vue';
// import { useTableSort } from '/@/hooks/useTableSort';
// import { useValidateUniqueness } from '/@/hooks/useValidateUniqueness';
import { deepClone } from '/@/utils/other';

import { userSexMap } from '../types';
import * as userApi from '/@/api/auth/user';
import { formatDate } from '/@/utils/formatTime';

const props = defineProps(['item', 'roleList']);
const emit = defineEmits(['update', 'insert']);
//#region ====================== 增加、修改记录操作, dialog init======================
const isEditDialog = ref(false);
const dialogTitle = computed(() => {
	return isEditDialog.value ? `修改用户【${props.item?.user_name}】` : `添加用户`;
});
const dialogHeaderIcon = computed(() => {
	return isEditDialog.value ? 'ele-Edit' : 'ele-Plus';
});
const dialogFormValue = ref(null);
const dialogIsShow = defineModel({
	type: Boolean,
});
const dialogFormRef = ref<FormInstance>(null);
const validatePassword = (_rule: any, value: any, callback: any) => {
	if (!dialogFormValue.value.password) {
		callback('请先输入新密码！');
	} else if (dialogFormValue.value.password !== dialogFormValue.value.confirmPassword) {
		callback(new Error('两次密码不一致！'));
	} else {
		callback();
	}
};
const dialogFormRules = ref<FormRules>({
	user_name: [{ required: true, message: '请输入用户ID', trigger: 'blur' }],
	password: [{ required: true, message: '请输入密码', trigger: 'blur' }],
	confirmPassword: [{ required: true, validator: validatePassword, trigger: 'blur' }],
});
const openOperateDialog = (row?) => {
	if (row) {
		isEditDialog.value = true;
		const { user_id, phone, real_name, part, sex, email, note } = row;
		dialogFormValue.value = deepClone({ user_id, phone, real_name, part, sex, email, note });
	} else {
		isEditDialog.value = false;

		dialogFormValue.value = {
			user_name: '',
			password: '',
			confirmPassword: '',
			phone: null,
			real_name: null,
			part: null,
			sex: null,
			email: null,
			note: null,
			user_roles: [],
		};
	}
};
const closeDialog = () => {
	dialogIsShow.value = false;
	dialogFormRef.value.clearValidate();
};

const submitFormValue = async () => {
	const valid = await dialogFormRef.value.validate().catch(() => {});
	if (!valid) return;
	const updateTime = formatDate(new Date());

	if (isEditDialog.value) {
		const res = await userApi.updateUserInfoByPost(dialogFormValue.value);
		emit('update', { ...dialogFormValue.value });

		closeDialog();
		ElMessage.success('修改用户成功');
	} else {
		const sendForm = { ...dialogFormValue.value, user_roles: dialogFormValue.value.user_roles?.join(',') };

		const res = await userApi.addUserInfoByPost(sendForm);
		const newData = {
			user_id: res.user_id,
			...dialogFormValue.value,
			create_time: updateTime.slice(0, 10),
		};
		emit('insert', newData);
		// tableData.value.push(newData);
		closeDialog();
		ElMessage.success('添加用户成功');
	}
};

//#endregion

watch(
	() => dialogIsShow.value,
	(val) => {
		if (!val) return;
		openOperateDialog(props.item);
	}
);
</script>
<style scoped lang="scss"></style>
